%% Velocity based on Rozenbaum, JETP Lett. 2009, and personal communication, 2017
% Ofer Kedem, Northwestern University 2017
% Contact at www.oferkedem.com
% Kedem, Lau and Weiss, Nano Lett., 2017

clear all
% General stuff
cross_section = 500e-6*50e-9; % [m^2] Actual dimensions of source/drain electrodes
channel_length = 12e-6; % [m]
electron_density = 1e21; %[1/m^3] Bisquert, Chem Phys Lett 2008 says 1e18
n = electron_density;
q = 1.6e-19; % [C]
filename = 'Simulation for D21A 50 pc P3HT, D 25e-9';

% Spatial potential
V1 = 0.25*q; % [J]
V2 = 0.05*q; % [J]

% Temporal modulation
f1 = 1; % [dimensionless]
alpha = -0.9; % [dimensionless] Selected so that the current inversion matches experiments
delta = (0.05:0.05:0.95)'; % [dimensionless]
duty_ratio = 1-delta;

% General parameters
kB = 1.38e-23; %[J/K]
T = 293; %[K]
beta = 1/(kB*T);
D = 1e-8; % [m^2*sec^-1] Bisquert, Chem Phys Lett 2008 says 2e-5 cm^2/sec = 2e-9 m^2/sec, 
            % but this produces really low resonance frequency, similar to the P3HT experiments
L = 800e-9; % [m] Actual periodicity of finger electrodes
%logs = 4:0.02:7;
%f = 10.^logs';
f = 1e3:1e3:1000e3;
tau = 1./f; % 1/gamma, 1/frequency [sec]
xi = (L/(2*pi))^2./(D.*tau); % [dimensionless]

% Stochastic functions
z = (xi./(4.*delta.*(1-delta))); % [dimensionless]
phi1_stoc = (3.*xi.*(1+2.*z))./((1+4*z).^2.*(1+z)); % [dimensionless]
phi2_stoc = (1-2*delta).*6.*xi.*z./((1+4*z).^2.*(1+z)); % [dimensionless]

% Deterministic functions
n_elem = 20;
phi1_d = phi1_det(xi,delta,n_elem); % [dimensionless]
phi2_d = phi2_det(xi,delta,n_elem); % [dimensionless]

% Choosing the function
phi1 = phi1_d;
phi2 = phi2_d;

% Ratchet velocity
v = (pi/4)*(D/L)*beta^3*V1^2*V2.*((1-alpha).^2).*...
    ((1+alpha).*phi1+(1-alpha).*phi2); % [m/s]
I = v.*electron_density.*cross_section.*q;

%% Export figure

figure;
LB    = flipud(lbmap(256,'BrownBlue'));
colormap(LB);
imagesc(f/1000,duty_ratio,I'/1e-9);
hold on
% Find peak frequency per duty ratio
[max_currents, max_f_indices] = max(I',[],2);
f_peak = f(max_f_indices)';
scatter(f_peak/1000, duty_ratio,200,'.','white');
hold off
xlim([25,1025]);
ylim([0.025,0.975]);
h = colorbar;
maxcolor = max(abs([min(I/1e-9),max(I/1e-9)]));
caxis([-maxcolor maxcolor]);
%maxcolor = 14;
ylabel('$\mathbf{Duty}$ $\mathbf{ratio}$','FontSize',24,'Interpreter','latex');
xlabel('$\mathbf{f [kHz]}$','FontSize',24,'Interpreter','latex');
h.Title.String = '$\mathbf{I_{sc} [nA]}$';
h.Title.Interpreter = 'latex';
set(h,'FontSize',24);
set(gca,'FontSize',24);
view(0,-90);
fig                     = gcf;
fig.InvertHardcopy      = 'off';
fig.PaperUnits          = 'inches';
fig.PaperPosition       = [0 0 8 6.5];
fig.PaperPositionMode = 'manual';
set(gcf, 'color', [1 1 1]);
set(gca, 'XTick', [200 400 600 800 1000]);
% print(gcf,'-dpng','-r300',filename)
% close(gcf)

%% Current vs. duty ratio plot

temp_curr = I(2,:)';
figure;
plot(duty_ratio,temp_curr/1e-9);

ax = gca;
set(ax,'FontSize',16);
xlabel('Duty ratio','FontSize',16);
ylabel('I_{sc} (nA)','FontSize',16);

%% Current vs. frequency plot

figure;
plot(f'/1000,I(:,1)'/1e-9);

ax = gca;
set(ax,'FontSize',16);
xlabel('f (kHz)','FontSize',16);
ylabel('I_{sc} (nA)','FontSize',16);
%ax.XScale = 'log';

[M,index] = max(I(:,1)');
f_peak = f(index);
s = f_peak / D;
%% Current vs. dimensionless frequency plot, at 0.5 duty ratio

figure;
plot(xi',I(:,10)'/1e-9);

ax = gca;
set(ax,'FontSize',16);
xlabel('xi','FontSize',16);
ylabel('I_{sc} (nA)','FontSize',16);

%find the dimensionless freq xi that maximizes Isc, for duty ratio 0.5
[M,index] = max(I(:,10)');
xi_peak = xi(index);
cross_section/channel_length

f_vs_cond_slope = 4*pi^2*xi_peak*kB*T/(L^2*q^2*n)

D_calc = 30e-3*kB*T/(q^2*2e23)